//------------------------------------------------------------------------------
// pvt performance visualization toolkit
//
// Copyright (c) 2014-2016 RWTH Aachen University, Germany,
// Virtual Reality & Immersive Visualisation Group.
//------------------------------------------------------------------------------
//                                License
//
// This library is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// In the future, we may decide to add a commercial license
// at our own discretion without further notice.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#ifndef PVT_DIRECTED_VARIATION_HPP_
#define PVT_DIRECTED_VARIATION_HPP_

#include <algorithm>
#include <cmath>
#include <vector>

#include <pvt/api.hpp>

namespace pvt {

class severity_view;

//------------------------------------------------------------------------------
class PVT_API directed_variation
{
public:
  directed_variation(std::size_t num_dimensions)
  : m_directed_variation(num_dimensions, 0.0)
  , m_mean              (0.0)
  , m_variance          (0.0)
  {
    
  }
  
  double get_mean         () const
  {
    return m_mean;
  } 
  double get_variance     () const
  {
    return m_variance;
  }
  double get_std_deviation() const
  {
    return sqrt(m_variance);
  }
  
  double&       at(std::size_t index)
  {
    return m_directed_variation.at(index);
  }
  const double& at(std::size_t index) const
  {
    return m_directed_variation.at(index);
  }
  
  double&       operator[](std::size_t index)
  {
    return m_directed_variation[index];
  }
  const double& operator[](std::size_t index) const
  {
    return m_directed_variation[index];
  }
  
  std::vector<double>::iterator       begin()
  {
    return m_directed_variation.begin();
  }
  std::vector<double>::const_iterator begin() const
  {
    return m_directed_variation.begin();
  }
  
  std::vector<double>::iterator       end()
  {
    return m_directed_variation.end();
  }
  std::vector<double>::const_iterator end() const
  {
    return m_directed_variation.end();
  }

  std::size_t size() const
  {
    return m_directed_variation.size();
  }

  void set_to_zero()
  {
    std::for_each(m_directed_variation.begin(),
                  m_directed_variation.end(),
                  [](double& v) { v = 0.0; });
    m_mean = 0.0;
    m_variance = 0.0;
  }

  void compute(const severity_view& severity_view);
  void dump   (const std::string&   filename) const;
  bool read   (const std::string&   filename,
               const severity_view& severity_view);
private:
  std::vector<double> m_directed_variation;
  double              m_mean;
  double              m_variance;
};

} // namespace pvt

#endif // #ifndef PVT_DIRECTED_VARIATION_HPP_
