//------------------------------------------------------------------------------
// pvt performance visualization toolkit
//
// Copyright (c) 2014-2016 RWTH Aachen University, Germany,
// Virtual Reality & Immersive Visualisation Group.
//------------------------------------------------------------------------------
//                                License
//
// This library is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// In the future, we may decide to add a commercial license
// at our own discretion without further notice.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include <cassert>
#include <cstdio>

#include <pvt/cnode.hpp>
#include <pvt/directed_variation.hpp>
#include <pvt/fft.hpp>
#include <pvt/fft_statistics.hpp>
#include <pvt/severity_view.hpp>

namespace pvt {

//------------------------------------------------------------------------------
void
directed_variation::compute
(const severity_view& severity_view)
{
  /// \todo Determine what to do with multiple selections
  
  set_to_zero();

  assert(severity_view.is_valid());

  fft fft(severity_view.get_performance_data()->
          get_cartesian_topologies()[0].get_index());
    
  severity_view.get_performance_data()->get_fft(severity_view, fft);
  
  m_mean     = fft.get_mean    ();
  m_variance = fft.get_variance();
  
  fft.get_directed_variation(*this);
}


//------------------------------------------------------------------------------
void
directed_variation::dump
(const std::string& filename) const
{
  auto file = fopen(filename.c_str(), "wb");
  for (auto value : m_directed_variation)
    fwrite(&value, sizeof(double), 1, file);
  fclose(file);
}


//------------------------------------------------------------------------------
bool
directed_variation::read
(const std::string&   filename,
 const severity_view& severity_view)
{
  auto file = fopen(filename.c_str(), "rb");
  const auto success = (file != nullptr);
  if (file != nullptr)
  {
    fread(m_directed_variation.data(),
          m_directed_variation.size() * sizeof(double),
          1,
          file);
  }
  else
  {
    for_each(m_directed_variation.begin(),
             m_directed_variation.end  (),
             [](double& v){ v = 0.0; });
  }
  fclose(file);
  
  fft_statistics fft_statistics;
  severity_view.get_performance_data()->get_fft_statistics(severity_view,
                                                           fft_statistics);
  m_mean     = fft_statistics.get(fft_statistics::mean);
  m_variance = fft_statistics.get(fft_statistics::variance);
  
  return success;
}

} // namespace pvt
