//------------------------------------------------------------------------------
// pvt performance visualization toolkit
//
// Copyright (c) 2014-2016 RWTH Aachen University, Germany,
// Virtual Reality & Immersive Visualisation Group.
//------------------------------------------------------------------------------
//                                License
//
// This library is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// In the future, we may decide to add a commercial license
// at our own discretion without further notice.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include <iostream>

#include <pvt/suitable_3d_view_computer.hpp>

namespace pvt {

//------------------------------------------------------------------------------
bool
suitable_3d_view_computer::is_suitable_3d_view
(const directed_variation& directed_variation)
const
{
  if (directed_variation.get_mean() == 0.0)
    return false;
  
  double filter = 0.0;
  for (std::size_t i = 0; i < m_filter_dimension_enabled.size(); ++i)
  {
    filter += static_cast<double>(m_filter_dimension_enabled[i]) *
              (i == m_filter_dimension_enabled.size() - 1 ? 2.0 : 1.0) *
              directed_variation[i] * directed_variation[i];
  }
  
  return directed_variation.get_std_deviation() / directed_variation.get_mean()
         - sqrt(filter) >= m_threshold;
}

} // namespace pvt
